/*-------------------------------------------------------------------
 *
 *  @file location.c
 *
 *  @author Collin R. Mulliner <collin@mulliner.org>
 *
 *  (c) Collin R. Mulliner
 *
 *  This is part of the Bluetooth Location Tracker (BLT) project.
 *
 *-------------------------------------------------------------------*/

/*
 * This file is part of blt_server
 *
 * blt_server is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * blt_server is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with blt_server; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
 
#include "blt_server.h"

int blt_ln_getlocation(struct blt_loc_names_t *ln, char *login, struct blt_location_name_t *loc)
{
	int retval = 0;
	int i;
	

	assert(ln != NULL);
	
	pthread_rwlock_rdlock(&ln->lock);
	
	for (i = 0; i < ln->numLocations; i++) {
		if (memcmp(ln->locs[i].login, login, strlen(ln->locs[i].login)) == 0) {
			memcpy(loc, &ln->locs[i], sizeof(struct blt_location_name_t));
			retval = 1;
			break;
		}
	}
	
	pthread_rwlock_unlock(&ln->lock);
	
	return(retval);
}

int blt_ln_load(struct blt_loc_names_t *ln)
{
	FILE *fp;
	char buffer[1024];
	struct blt_location_name_t loc;
	struct blt_location_name_t *tmp;
	char *p;
	unsigned int maxLocs = 0;
	
	
	if ((fp = fopen(BLT_LOCATION_NAMES_FILE, "r")) == NULL) {
		return(0);
	}
	
	/* lock */
	pthread_rwlock_wrlock(&ln->lock);
	
	/* destroy old data */
	blt_ln_destroy(ln);
	
	while (!feof(fp)) {
		memset(buffer, 0, 1024);
		memset(&loc, 0, sizeof(struct blt_location_name_t));
		fgets(buffer, 1024, fp);
		
		if (buffer[0] == '#') continue;
		
		sscanf(buffer, "%d %d", &loc.la, &loc.lg);
		
		p = strstr(buffer, "\"");
		if (p > 0) strcpyendchr(p+1, loc.name, '"', BLT_LOCATION_NAME_LENGTH+1);
		else continue;
		p = p + strlen(loc.name)+2;
		p = strstr(p, "\"");
		if (p > 0) strcpyendchr(p+1, loc.devname, '"', BLT_LOCATION_NAME_LENGTH+1);
		else continue;
		p = p + strlen(loc.devname)+2;
		p = strstr(p, "\"");
		if (p > 0) strcpyendchr(p+1, loc.login, '"', 20);
		else continue;
		p = p + strlen(loc.login)+2;
		p = strstr(p, "\"");
		if (p > 0) strcpyendchr(p+1, loc.password, '"', 20);
		else continue;
		
		#ifdef DEBUG
		fprintf(stderr, "name: %s\n", loc.name);
		fprintf(stderr, "device name: %s\n", loc.devname);
		fprintf(stderr, "lat: %d log: %d\n", loc.la, loc.lg);
		fprintf(stderr, "login: %s\n", loc.login);
		fprintf(stderr, "password: %s\n", loc.password);
		#endif
		
		if (ln->numLocations+1 > maxLocs) {
			tmp = ln->locs;
			ln->locs = malloc((ln->numLocations+10) * sizeof(struct blt_location_name_t));
			if (tmp != NULL) memcpy(ln->locs, tmp, ln->numLocations * sizeof(struct blt_location_name_t));
			maxLocs += 10;
		}
		memcpy(&ln->locs[ln->numLocations], &loc, sizeof(struct blt_location_name_t));
		ln->numLocations++;
	}
	
	/* unlock */
	pthread_rwlock_unlock(&ln->lock);
	
	fclose(fp);
	
	return(1);
}

void blt_ln_destroy(struct blt_loc_names_t *ln)
{
	assert(ln != NULL);

	if (ln->locs != NULL) {
		free(ln->locs);
		ln->locs = NULL;
	}
	ln->numLocations = 0;
}

void blt_ln_init(struct blt_loc_names_t *ln)
{
	assert(ln != NULL);
	
	ln->locs = NULL;
	ln->numLocations = 0;
	
	pthread_rwlock_init(&ln->lock, NULL);
}

void blt_ln_deinit(struct blt_loc_names_t *ln)
{
	assert(ln != NULL);
	
	blt_ln_destroy(ln);
	
	pthread_rwlock_destroy(&ln->lock);
}
